/* $Header$
 *
 * alarmingdns.c - versions of DNS routines with timeouts.  Actually,
 * it's really only the host ones that in practice can hang,
 * but once you start doing this stuff it's hard to stop.
 */

/* What can I say?  I'm a Berkeley boy... */
#define _BSD_SOURCE
#include <features.h>
#include "uincludes.h"
#include <stdarg.h>

#include <signal.h>
#include <setjmp.h>
#include "alarmingdns.h"

static jmp_buf env;

static void
DNS_Timeout(int signo)
{
	longjmp(env, signo);
}

/* This controls how long (in seconds) we wait for a response before
 * timing out.  5 seconds might be too fast in some environments, but
 * should generally be tolerable.
 */
#define ALARMTIME	5

/* This is simply a debug routine to ensure the alarm stuff is working
 * properly.  If you ever see a message from it, let me know!
 */
static void
myalarm(int howlong)
{
	static time_t was;
	int slow;

	alarm(howlong);
	if (howlong)
		was = time(NULL);
	else {
		slow = time(NULL)-was;
		if (slow > ALARMTIME) {
			fprintf(stderr,
				"myalarm: didn't timeout after %d seconds!!\n",
				slow);
			/* abort(); */
		}
	}
}

struct protoent *
alarmgetprotobyname(const char *name, int timeout)
{
	/* Linux getprotobyname only reads /etc/protocols, so... */
	return getprotobyname(name);
}

struct protoent *
alarmgetprotobynumber(int protno, int timeout)
{
	/* Linux getprotobynumber only reads /etc/protocols, so... */
	return getprotobynumber(protno);
}


struct hostent *
alarmgethostbyname(char *name, int timeout)
{
	struct hostent *answer;
	void (*was_sig)(int);

	if (!timeout) return gethostbyname(name);

	was_sig = signal(SIGALRM, DNS_Timeout);

	if (setjmp(env)) {
		fprintf(stderr, "alarmgethostbyname(%s): time out\n", name);
		(void) signal(SIGALRM, was_sig);
		return NULL;
	}
	myalarm(ALARMTIME);
	answer = gethostbyname(name);
	myalarm(0);
	(void) signal(SIGALRM, was_sig);
	return answer;
}


struct hostent *
alarmgethostbyaddr(char *addr, int addrsize, int family, int timeout)
{
	struct hostent *answer;
	void (*was_sig)(int);

	if (!timeout) return gethostbyaddr(addr, addrsize, family);

	was_sig = signal(SIGALRM, DNS_Timeout);

	if (setjmp(env)) {
		char buffer[50];
		struct in_addr one;

		one.s_addr = *(__u32 *)addr;
		strcpy(buffer, inet_ntoa(one));
		fprintf(stderr, "alarmgethostbyaddr(%s): time out\n", buffer);
		(void) signal(SIGALRM, was_sig);
		return NULL;
	}
	myalarm(ALARMTIME);
	answer = gethostbyaddr(addr, addrsize, family);
	myalarm(0);
	(void) signal(SIGALRM, was_sig);
	return answer;
}

struct servent *
alarmgetservbyname(const char *name, const char *protoname, int timeout)
{
	/* Linux getservbyname only reads /etc/services, so... */
	return getservbyname(name, protoname);
}

struct servent *
alarmgetservbyport(int port, const char *protoname, int timeout)
{
	/* Linux getservbyport only reads /etc/services, so... */
	return getservbyport(port, protoname);
}
